﻿using System.Collections.ObjectModel;
using System.ComponentModel;

namespace TestProject
{
    /// <summary>
    /// 경매 항목
    /// </summary>
    public class AuctionItem : INotifyPropertyChanged
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 속성 변경시 이벤트 - PropertyChanged

        /// <summary>
        /// 속성 변경시 이벤트
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 설명
        /// </summary>
        private string description;

        /// <summary>
        /// 이미지 컬렉션
        /// </summary>
        private ObservableCollection<string> imageCollection;

        /// <summary>
        /// 특징 타입
        /// </summary>
        private FeatureType featureType;

        /// <summary>
        /// 현재가
        /// </summary>
        private int currentPrice;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 설명 - Description

        /// <summary>
        /// 설명
        /// </summary>
        public string Description
        {
            get
            {
                return this.description;
            }
            set
            {
                this.description = value;

                FirePropertyChangedEvent(nameof(Description));
            }
        }

        #endregion
        #region 이미지 컬렉션 - ImageCollection

        /// <summary>
        /// 이미지 컬렉션
        /// </summary>
        public ObservableCollection<string> ImageCollection
        {
            get
            {
                return this.imageCollection;
            }
        }

        #endregion
        #region 특징 타입 - FeatureType

        /// <summary>
        /// 특징 타입
        /// </summary>
        public FeatureType FeatureType
        {
            get
            {
                return this.featureType;
            }
            set
            {
                this.featureType = value;

                FirePropertyChangedEvent(nameof(FeatureType));
            }
        }

        #endregion
        #region 현재가 - CurrentPrice

        /// <summary>
        /// 현재가
        /// </summary>
        public int CurrentPrice
        {
            get
            {
                return this.currentPrice;
            }
            set
            {
                this.currentPrice = value;

                FirePropertyChangedEvent(nameof(CurrentPrice));
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - AuctionItem()

        /// <summary>
        /// 생성자
        /// </summary>
        public AuctionItem()
        {
            this.imageCollection = new ObservableCollection<string>();
        }

        #endregion
        #region 생성자 - AuctionItem(description, featureType, currentPrice)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="description">설명</param>
        /// <param name="featureType">특징 타입</param>
        /// <param name="currentPrice">현재가</param>
        public AuctionItem(string description, FeatureType featureType, int currentPrice)
        {
            this.description     = description;
            this.imageCollection = new ObservableCollection<string>();
            this.featureType     = featureType;
            this.currentPrice    = currentPrice;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 속성 변경시 이벤트 발생시키기 - FirePropertyChangedEvent(propertyName)

        /// <summary>
        /// 속성 변경시 이벤트 발생시키기
        /// </summary>
        /// <param name="propertyName">속성명</param>
        protected void FirePropertyChangedEvent(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion
    }
}